@@ -25,10 +25,8 @@
 #include "svn_types.h"
 #include "svn_delta.h"
 #include "svn_fs.h"
-#include "svn_md5.h"
-#include "svn_path.h"
 #include "svn_repos.h"
-#include "svn_pools.h"
+#include "svn_private_config.h"
 
 
 /*** Backstory ***/
@@ -158,7 +156,7 @@ path_driver_cb_func (void **dir_baton,
       if ((kind != svn_node_dir) && (kind != svn_node_file))
         return svn_error_createf 
           (SVN_ERR_FS_NOT_FOUND, NULL, 
-           "Filesystem path '%s' is neither a file nor a directory", path);
+           _("Filesystem path '%s' is neither a file nor a directory"), path);
     }
 
   /* Handle any adds/opens. */
